/*
PRICE was written by Graham Ruby.
This file is part of PRICE.

PRICE is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

PRICE is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PRICE.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PARAMETERIZEDREADFILE_CPP
#define PARAMETERIZEDREADFILE_CPP

#include "ParameterizedReadFile.h"
#include "AssemblyException.h"
#include <iostream>
using namespace::std;

// constructors
ParameterizedReadFile::ParameterizedReadFile(){}

ParameterizedReadFile::ParameterizedReadFile(ReadFile* readFile, ParamsMapping* pm) :
  _initialCycle(0),
  _totalCycles(-1)
{
  _readFile = readFile->copy();
  _pm = new ParamsMapping(pm);
}
ParameterizedReadFile::ParameterizedReadFile(ReadFile* readFile, ParamsMapping* pm, int initialCycle) :
  _initialCycle(initialCycle),
  _totalCycles(-1)
{
  _readFile = readFile->copy();
  _pm = new ParamsMapping(pm);
}
ParameterizedReadFile::ParameterizedReadFile(ReadFile* readFile, ParamsMapping* pm, int initialCycle, int numCycles) :
  _initialCycle(initialCycle),
  _totalCycles(numCycles)
{
  _readFile = readFile->copy();
  _pm = new ParamsMapping(pm);
}


ParameterizedReadFile::~ParameterizedReadFile(){
  delete _readFile;
  delete _pm;
}


void ParameterizedReadFile::splitFile(set<ReadFile*>* putFilesHere, int numFiles){
  set<ReadFile*> splitFiles;
  _readFile->splitFile(&splitFiles, numFiles);
  for (set<ReadFile*>::iterator it = splitFiles.begin(); it != splitFiles.end(); ++it){
    putFilesHere->insert( new ParameterizedReadFile( *it, _pm, _initialCycle, _totalCycles) );
    delete *it;
  }
}
void ParameterizedReadFile::splitFile(set<ParameterizedReadFile*>* putFilesHere, int numFiles){
  set<ReadFile*> splitFiles;
  _readFile->splitFile(&splitFiles, numFiles);
  for (set<ReadFile*>::iterator it = splitFiles.begin(); it != splitFiles.end(); ++it){
    putFilesHere->insert( new ParameterizedReadFile( *it, _pm, _initialCycle, _totalCycles) );
    delete *it;
  }
}
ReadFile* ParameterizedReadFile::subFile(long numReadsSkip, long numReadsKeep){
  ReadFile* innerSubFile = _readFile->subFile(numReadsSkip, numReadsKeep);
  ReadFile* subFile = new ParameterizedReadFile( innerSubFile, _pm, _initialCycle, _totalCycles);
  delete innerSubFile;
  return subFile;
}
ParameterizedReadFile* ParameterizedReadFile::copy(){
  return new ParameterizedReadFile( _readFile, _pm, _initialCycle, _totalCycles);
}


bool ParameterizedReadFile::useThisCycle(int cycleNum){
  if (cycleNum < _initialCycle){ return false; }
  else {
    int finalCycle;
    if (_totalCycles == -1){ finalCycle = cycleNum; }
    else { finalCycle = _initialCycle + _totalCycles - 1; }
    return cycleNum <= finalCycle;
  }
}

ParamsMapping* ParameterizedReadFile::getMapParams(){
  return new ParamsMapping(_pm);
}


// inherited interface from ReadFile; these are forwarded to the carried file
string ParameterizedReadFile::getName(){ return _readFile->getName(); }
void ParameterizedReadFile::open(){ _readFile->open(); }
void ParameterizedReadFile::open(bool getBothEnds){ _readFile->open(getBothEnds); }
void ParameterizedReadFile::openWithCarriers(int numSets){ _readFile->openWithCarriers(numSets); }
void ParameterizedReadFile::openWithCarriers(int numSets, bool getBothEnds){ _readFile->openWithCarriers(numSets,getBothEnds); }
void ParameterizedReadFile::openNormalized(){ _readFile->openNormalized(); }
void ParameterizedReadFile::openNormalized(bool getBothEnds){ _readFile->openNormalized(getBothEnds); }
void ParameterizedReadFile::openNormWithCarriers(int numSets){ _readFile->openNormWithCarriers(numSets); }
void ParameterizedReadFile::openNormWithCarriers(int numSets, bool getBothEnds){ _readFile->openNormWithCarriers(numSets,getBothEnds); }
bool ParameterizedReadFile::hasRead(){ return _readFile->hasRead(); }
ScoredSeq* ParameterizedReadFile::getRead(){ return _readFile->getRead(); }
void ParameterizedReadFile::skipRead(){ _readFile->skipRead(); }
void ParameterizedReadFile::skipReads(long numToSkip){ _readFile->skipReads(numToSkip); }
void ParameterizedReadFile::close(){ _readFile->close(); }
long ParameterizedReadFile::numReads(){ return _readFile->numReads(); }
long ParameterizedReadFile::ampliconSize(){ return _readFile->ampliconSize(); }

void ParameterizedReadFile::bufferQueue(Read * r){ throw AssemblyException::CallingError("PRF::bufferQueue"); }
void ParameterizedReadFile::bufferUnqueue(Read * r){ throw AssemblyException::CallingError("PRF::bufferUnqueue"); }
void ParameterizedReadFile::bufferFill(){ throw AssemblyException::CallingError("PRF::bufferFill"); }
void ParameterizedReadFile::updateBuffer(){ throw AssemblyException::CallingError("PRF::updateBuffer"); }
void ParameterizedReadFile::emptyBuffer(){ throw AssemblyException::CallingError("PRF::emptyBuffer"); }


#endif
